#! /bin/sh

#-----------------------
# configured variables
agent_conf="/etc/drweb/agent.conf"
monitor_conf="/etc/drweb/monitor.conf"
monitor_enable_conf="/etc/drweb/drweb-monitor.enable"
agent_bin="/opt/drweb/drweb-agent"
version="6.0.2.3"

# ----------------------

usage="$0 [-h] [-v] [-e [yes|no]] [-a ES-address] [-p ES-port] [-k ES-public-key] [-s agent-parameter]"

set -e

phelp()
{
    cat <<EOF
    agent-config.sh: change agent.conf
Usage: $usage
    -e ES mode
    -a sets DrWeb ES server address
    -p sets DrWeb ES server port
    -k sets ES server public key
    -r restart Agent if config has been changed
    -s show value
    -d drop login
    -v show version
    -h print help
EOF
}

usage()
{
    echo "Usage: $usage" >&2
}

[ -z "$1" ] && { usage; exit 2; }

show()
{
    conf=$1
    param=$2;
    cat $conf | sed -n "s/^ *$param *= *//p"
}

restart_monitor()
{
    pidfile="/var/drweb/run/drweb-monitor.pid";
    if [ -f $pidfile ];then
	pid=$(head -1 $pidfile);
	kill $pid;

	TIMEOUT=10;
	TIME=0;
	while [ $TIME -lt $TIMEOUT -a -f $pidfile ];do
            TIME=$(($TIME + 1))
            sleep 1
	done
	if [ -f $pidfile ];then
	    echo "drweb-monitor: cannot stop" >&2;
	    exit 1
	fi
    fi

    /opt/drweb/drweb-monitor;
}

while getopts "k:e:a:p:hvrds:" opt; do
    case "$opt" in
	a)	address=$OPTARG;;
	e)      es_mode=$OPTARG;;
	d)      drop_login=yes;;
	p)	port=$OPTARG;;
	k)      key=$OPTARG;;
	r)      restart=1;;
	s)      show $agent_conf $OPTARG;;
	v)      echo "Version: $version"; exit 0;;
	h)      phelp; exit 0;;
	*)	usage; exit 2;;
    esac
done
shift $(( $OPTIND - 1 ))
[ -z "$1" ] || { usage; exit 2; }

conf_changed=0

set_param()
{
    conf=$1;
    param=$2;
    value=$3;

    tmp=$(mktemp);
    cat $conf | sed "s/^ *$param *=.*/$param = $value/" > $tmp;

    if [ x"$conf_changed" = x"0" ];then
	status=0
	cmp -s $tmp $conf || status=$?;
	if [ x"$status" = x"1" ];then
	    conf_changed=1;
	fi
    fi

    # make backup
    [ -e $conf.old ] || cp $conf $conf.old;

    mv -f $tmp $conf;
    chmod 644 $conf;
}

fix_monitor_conf()
{
    echo ENABLE=1 > $monitor_enable_conf

    conf=$monitor_conf;
    run_app_list=$(show $conf RunAppList);
    if (echo $run_app_list | grep AGENT >/dev/null);then
	return;
    fi

    if [ x"$run_app_list" = x"" ] || ([ x"$run_app_list" = x"MAILD" ] && !((ls /etc/drweb/monitor/maild*.mmc 2>&1) >/dev/null));then
	run_app_list="AGENT"
    else
	run_app_list="AGENT, "$run_app_list
    fi

    set_param $conf RunAppList "$run_app_list"
}

[ -z "$address" ] || set_param $agent_conf ServerHost $address
[ -z "$port" ] || set_param $agent_conf ServerPort $port
[ -z "$es_mode" ] || set_param $agent_conf UseEnterpriseMode $es_mode

key_dest=$(show $agent_conf PublicKeyFile)

status=0

if [ x"$key" != x"" ];then
    cmp -s "$key_dest" "$key" || status=$?;
fi

if [ x"$status" != x"0" ] && [ -e "$key" ];then
    conf_changed=1;
    [ -z "$key" ] || cp --backup=t "$key" "$key_dest" && chmod a+r "$key_dest"
fi

if [ x"$drop_login" = x"yes" ];then
    $agent_bin -d;
fi

if [ x"$restart" = x"1" ];then
    fix_monitor_conf;
    restart_monitor;
fi
